// PacketEchoServerDLL.cpp : Defines the exported functions for the DLL application.
//

#define WASP_MAP_W_AND_A

#include "WASPPluginSDK\WASPDLLEntryPoints.h"

#include "ServerInstance.h"

#pragma comment(lib, "ws2_32.lib")

// Message framing

unsigned long __cdecl GetMinimumMessageSize()
{
   return sizeof(int);
}

unsigned long __cdecl GetMessageSize(
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData,
   const BYTE * const pData,
   const WaspDataLength dataLength,
   const WaspDataLength previousLength)
{
   (void)connectionHandle;
   (void)connectionUserData;
   (void)dataLength;
   (void)previousLength;

   return ntohl(*reinterpret_cast<const int *>(pData)) + sizeof(int);
}

// Message handling

WaspInstanceHandle __cdecl Initialise(
   WaspFunctions *pWaspFunctions,
   const char *pConfigValue)
{
   if (pWaspFunctions->size < sizeof(WaspFunctions))
   {
      // By testing to see that the supplied function table is
      // larger than or equal to our function table we can 
      // return an error which indicates that we failed to
      // initialise if we have been built expecting a later 
      // (and therefore larger) function table than the version 
      // of WASP that is hosting us is supplying us.

      return 0;
   }

   WaspInstanceHandle handle = 0;

   try
   {
      CServerInstance *pServer = new CServerInstance(
         pConfigValue, 
         pWaspFunctions);

      handle = reinterpret_cast<WaspInstanceHandle>(pServer);
   }
   catch (...)
   {
      // mustn't let exceptions leak from our entry points...
   }

   return handle;
}

WaspConnectionUserData __cdecl OnConnectionEstablished(
   const WaspInstanceHandle instanceHandle,
   const WaspConnectionHandle connectionHandle,
   const sockaddr *pAddress,
   const WaspAddressLength addressLength)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   return pInstance->OnConnectionEstablished(
      connectionHandle,
      pAddress,
      addressLength);
}

void __cdecl OnReadCompleted(
   const WaspInstanceHandle instanceHandle,
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData,
   const BYTE * const pDataIn,
   const WaspDataLength dataLength)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   pInstance->OnReadCompleted(
      connectionHandle, 
      connectionUserData,
      pDataIn,
      dataLength);
}

void __cdecl OnConnectionClientClosed(
   const WaspInstanceHandle instanceHandle,
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   pInstance->OnConnectionClientClosed(connectionHandle, connectionUserData);
}

void __cdecl OnConnectionReset(
   const WaspInstanceHandle instanceHandle,
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData,
   const WaspLastError lastError)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   pInstance->OnConnectionReset(connectionHandle, connectionUserData, lastError);
}

void __cdecl OnConnectionClosed(
   const WaspInstanceHandle instanceHandle,
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData,
   const WaspConnectionClosureReason reason)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   pInstance->OnConnectionClosed(connectionHandle, connectionUserData, reason);
}

void __cdecl OnConnectionComplete(
   const WaspInstanceHandle instanceHandle,
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   pInstance->OnConnectionComplete(connectionHandle, connectionUserData);
}

void __cdecl OnShutdownInitiated(
   const WaspInstanceHandle instanceHandle)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   pInstance->OnShutdownInitiated();
}

void __cdecl Shutdown(
   const WaspInstanceHandle instanceHandle)
{
   CServerInstance *pInstance = reinterpret_cast<CServerInstance *>(instanceHandle);

   delete pInstance;
}

